#!/bin/sh

#################################################
# Programa creado por Haylem Candelario.	#
# Creador de isos booteables del sistema.	#
#		2013				#
#################################################


#----------- Variables del programa-----

KERNELS=" "
INITRDS=" "
ISOLINUX=" "

WORKDIR=`pwd` # Carpeta de trabajo
COMP=$WORKDIR/complementos #carpeta de complementos
RESISO=$WORKDIR/ISO
FOLDERS="$WORKDIR/SISTEMA/folders"

SQUASHFSOPTSHIGH="-no-recovery -always-use-fragments"

GENISO="$WORKDIR/complementos/genisoimage"
SQUASHFS="$WORKDIR/complementos/mksquashfs"
WHOAMI="$WORKDIR/complementos/whoami"

export LD_LIBRARY_PATH=$COMP/:LD_LIBRARY_PATH # Redireccionar donde se buscan las librerias.

ISO="isocrea.iso"
LIVECDLABEL="LinuxCRTISO"
#---------------------------------------



clear # Borrar la pantalla.



echo || echo || echo # Espacios adicionales.

echo "     #################################################"
echo "     #                                               #"
echo "     # Copyright Haylem Candelario Bauza 2013 GPLv2  #"
echo "     #             ISOCREA    HABANA.CUBA            #"
echo "     #                                               #"
echo "     #################################################"
echo

echo "Release:"
cat $COMP/release.txt

echo || echo || echo

if [ ! -z $1 ];then
 if [ -d $1 ];then
    WORKDIR=$1
    FOLDERS="$WORKDIR/SISTEMA/folders"
    echo "->   Se ha especificado otra carpeta de trabajo, usando: $WORKDIR como carpeta de trabajo."
    cp -vrp `pwd`/* $WORKDIR/
    echo
 else
    echo "->   Carpeta de trabajo especificada inexistente. Usando $WORKDIR" && echo
 fi
fi

if [ -d $WORKDIR/plugins/activados ];then
  if [ ! -z `ls $WORKDIR/plugins/activados/` ];then
    echo && echo "Ejecutando plugins disponibles..." && echo
    $WORKDIR/plugins/activados/*
    echo
  fi
fi

# Verificar si el usuario es root para ejecutar este programa.

testroot=`$WORKDIR/complementos/whoami` 

if [ "$testroot" != "root" ]; then
    echo && echo ":-( Debes ser root o usar sudo para continuar. Cancelado." && echo
    exit 1
fi

echo "->   Creando la estructura de carpetas ..."

mkdir $WORKDIR/ISO

mkdir $WORKDIR/SISTEMA
mkdir $WORKDIR/SISTEMA/folders
mkdir $WORKDIR/SISTEMA/folders/media
mkdir $WORKDIR/SISTEMA/folders/mnt
mkdir $WORKDIR/SISTEMA/folders/proc
mkdir $WORKDIR/SISTEMA/folders/sys

mkdir $WORKDIR/SISTEMA/ISOTMP
mkdir $WORKDIR/SISTEMA/ISOTMP/isolinux
mkdir $WORKDIR/SISTEMA/ISOTMP/live

echo " ">$WORKDIR/SISTEMA/ISOTMP/live/filesystem.packages # Crear vacio el .packages

cp $COMP/cp/memtest $WORKDIR/SISTEMA/ISOTMP/live/ # Agregar el kernel memtest

cp $COMP/splash/* $COMP/isolinux/ # Reemplazar la foto que aparece en el menu de boot 640x480.
cp $COMP/isolinux/* $WORKDIR/SISTEMA/ISOTMP/isolinux/ # Copiar la estructura isolinux

# Analizar kernels disponibles y crear menu principal

echo && echo "->   Copiando kernels e initrds ..." && echo

cp -v /boot/*linuz* $WORKDIR/SISTEMA/ISOTMP/live/
cp -v /boot/initrd* $WORKDIR/SISTEMA/ISOTMP/live/

KERNELS=`ls $WORKDIR/SISTEMA/ISOTMP/live/|grep linuz|cut -d "-" -f4`
INITRDS=`ls $WORKDIR/SISTEMA/ISOTMP/live/|grep initr|cut -d "-" -f4`

echo && echo "->   Analizando kernels disponibles y recortando sus nombres..."

for KERN in $KERNELS # Cambiar nombre largo de los kernels por sus nombres cortos el 686 es vmlinuz para 686
do
  mv $WORKDIR/SISTEMA/ISOTMP/live/vmlinuz*$KERN $WORKDIR/SISTEMA/ISOTMP/live/$KERN
done

for KERN in $KERNELS # Cambiar nombre largo de los kernels por sus nombres cortos el 686 es vmlinuz para 686 soporte pae y otros
do
  mv $WORKDIR/SISTEMA/ISOTMP/live/vmlinuz*$KERN* $WORKDIR/SISTEMA/ISOTMP/live/$KERN
done

echo "->   Analizando initrds disponibles y recortando sus nombres..." && echo

for INIT in $INITRDS
do
  mv $WORKDIR/SISTEMA/ISOTMP/live/init*$INIT $WORKDIR/SISTEMA/ISOTMP/live/$INIT.img
done

for INIT in $INITRDS # Soporte pae y otros
do
  mv $WORKDIR/SISTEMA/ISOTMP/live/init*$INIT* $WORKDIR/SISTEMA/ISOTMP/live/$INIT.img
done

#******************************************************************************
# Aqui se actualiza el archivo de menu de isolinux agregando
# Cada kernel con su initrd.

ISOLINUX=$WORKDIR/SISTEMA/ISOTMP/isolinux

echo && echo "->   Actualizando live.cfg de isolinux..." && echo

echo " ">$ISOLINUX/live.cfg
echo "label memtest">>$ISOLINUX/live.cfg
echo "   menu label Chequear memoria">>$ISOLINUX/live.cfg
echo "   kernel /live/memtest">>$ISOLINUX/live.cfg

echo " ">>$ISOLINUX/live.cfg
echo "label floppy">>$ISOLINUX/live.cfg
echo "   localboot 0x00">>$ISOLINUX/live.cfg

echo " ">>$ISOLINUX/live.cfg
echo "label disco1">>$ISOLINUX/live.cfg
echo "   localboot 0x80">>$ISOLINUX/live.cfg


mkdir $WORKDIR/temp
mv $WORKDIR/SISTEMA/ISOTMP/live/*.* $WORKDIR/temp
mv $WORKDIR/SISTEMA/ISOTMP/live/memtest $WORKDIR/temp

for KRN in `ls $WORKDIR/SISTEMA/ISOTMP/live/`
do
  echo && echo "->   Detectados: $KRN" && echo
  
 
     echo && echo "->   Enviado al archivo $KRN" && echo
     echo " ">>$ISOLINUX/live.cfg
     echo "label Linux $KRN">>$ISOLINUX/live.cfg
     echo "   kernel /live/$KRN">>$ISOLINUX/live.cfg
     echo "   append initrd=/live/$KRN.img boot=live config locales=es_ES.UTF-8 hostname=HaylemOS quiet">>$ISOLINUX/live.cfg

done

mv $WORKDIR/temp/* $WORKDIR/SISTEMA/ISOTMP/live/
rm -r $WORKDIR/temp

#******************************************************************************
# Instalando el instalador del sistema por defecto

echo && echo && echo "->   Poniendo instalador a tu sistema :-) (Necesita paquete python-tk)" && echo

dpkg -i $COMP/hcbinst.deb && echo




# Agregando las carpetas que no se copian al suqashfs
echo && echo "->   Agregando las carpetas vacias definidas en $FOLDERS/ que no se copian." && echo
$SQUASHFS $FOLDERS $WORKDIR/SISTEMA/ISOTMP/live/filesystem.squashfs -b 1M -no-duplicates $SQUASHFSOPTSHIGH
echo && echo && echo

# Agregando el resto del sistema

echo && echo "->   Agregando el resto del sistema (Puede tardar). :-)" && echo && echo
$SQUASHFS / $WORKDIR/SISTEMA/ISOTMP/live/filesystem.squashfs -b 1M -no-duplicates $SQUASHFSOPTSHIGH -e \
/.thumbails \
/.cache \
/media \
/mnt \
/proc \
/iso \
/etc/fstab \
/etc/mtab \
/sys \
`pwd`/

echo && echo && echo

# Creando el iso booteable


echo "->   Directorio de trabajo: $WORKDIR"
echo "->   Creando iso en $RESISO"

echo || echo || echo

   $GENISO    \
    -v \
    -r    \
    -V "$LIVECDLABEL"    \
    -cache-inodes    \
    -J    \
    -l    \
    -b isolinux/isolinux.bin    \
    -c isolinux/boot.cat    \
    -no-emul-boot    \
    -boot-load-size 4    \
    -boot-info-table    \
    -o $RESISO/$ISO "$WORKDIR/SISTEMA/ISOTMP" 
echo || echo || echo

# Conjunto de espacios finales

echo || echo || echo
